require_relative './helper'

class TestParser < Test::Unit::TestCase

  def raw_parse_line text
    p = Aster::Parser.new
    p.send :raw_parse_line, text
  end

  def parse_line text
    r = raw_parse_line text
    assert_equal false, r.garbage?
    r
  end

  def parse text
    p = Aster::Parser.new
    p.parse text
  end

  def text_indent(text)
    p = Aster::Parser.new
    p.send :text_indent, text
  end

  def test_leading_indent()
    assert_equal 0,   text_indent('')
    assert_equal 1,   text_indent('  ')
    assert_equal 2,   text_indent('    ')
    assert_equal -1,  text_indent('   ')
    assert_equal -1,  text_indent('     ')
  end

  def test_line_parse
    assert_equal ["a"], parse_line("a")

    tree = parse_line("a $a")
    assert_equal "a", tree.first
    assert_equal :a, tree.last.name

    assert_equal [:a, :b], parse_line("{$a $b}").first.map{|v| v.name.to_sym}
    assert_equal ["upload", "\"asdf.jpg\""], parse_line(%q(upload "asdf.jpg"))
    assert_equal ["upload", "nginx.conf"], parse_line("upload nginx.conf")
    assert_equal ["upload", "nginx/nginx.conf", "~/nginx.test"], parse_line("upload nginx/nginx.conf ~/nginx.test")
  end

  def test_parse_garbage

    def g p
      raw_parse_line(p).tap do |g|
        assert_equal true, g.garbage?
      end
    end


  end


  def test_command_parse
    c = parse "asdf one two"
    assert_equal c.first.function_name, :asdf
  end

  def test_asdf
    p = Aster::Parser.new
    tree = p.parse "if {not {test}}"
    command = tree.first
  end

  def test_parse
    parser = Aster::Parser.new
file = "
function test asdr
  exec test

function test2 asdf
  exec
"    
    tree = parser.parse file
  end

#   def test_ruby_script
#     tree = parse %Q{
# ruby
#   require 'rubygems'
#   a = ArchiveRecord::Base;
#   a.define "s" do |asdf|
#     asdf.asdf << #end
#   end
#     }
#     text = "require 'rubygems'\na = ArchiveRecord::Base;\na.define \"s\" do |asdf|\n  asdf.asdf << #end\nend\n  "
#     assert_equal text, tree.first.sub_text
#   end

end